# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.serialization import FieldMetadata
from ..core.unchecked_base_model import UncheckedBaseModel


class EditTextOptions(UncheckedBaseModel):
    """
    Text styling options for text fields
    """

    font_size: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="fontSize")] = pydantic.Field(
        alias="fontSize", default=None
    )
    """
    Font size in points
    """

    font_color: typing_extensions.Annotated[typing.Optional[typing.List[float]], FieldMetadata(alias="fontColor")] = (
        pydantic.Field(alias="fontColor", default=None)
    )
    """
    RGB color values (0-255) for text color
    """

    font: typing.Optional[str] = pydantic.Field(default=None)
    """
    Font family name
    """

    combing: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether this is a combed field (like SSN fields with individual character boxes)
    """

    max_length: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="maxLength")] = pydantic.Field(
        alias="maxLength", default=None
    )
    """
    Maximum number of characters allowed
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError as core_api_error_ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.request_options import RequestOptions
from ..core.serialization import convert_and_respect_annotation_metadata
from ..core.unchecked_base_model import construct_type
from ..errors.bad_request_error import BadRequestError
from ..errors.forbidden_error import ForbiddenError
from ..errors.internal_server_error import InternalServerError
from ..errors.not_found_error import NotFoundError
from ..errors.payment_required_error import PaymentRequiredError
from ..errors.too_many_requests_error import TooManyRequestsError
from ..errors.unauthorized_error import UnauthorizedError
from ..errors.unprocessable_entity_error import UnprocessableEntityError
from ..requests.extract_config_json import ExtractConfigJsonParams
from ..types.api_error import ApiError as types_api_error_ApiError
from ..types.extractor_version import ExtractorVersion
from ..types.max_page_size import MaxPageSize
from ..types.next_page_token import NextPageToken
from ..types.release_type import ReleaseType
from ..types.sort_dir import SortDir
from ..types.version_description import VersionDescription
from .types.extractor_versions_list_response import ExtractorVersionsListResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawExtractorVersionsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self,
        extractor_id: str,
        *,
        sort_dir: typing.Optional[SortDir] = None,
        next_page_token: typing.Optional[NextPageToken] = None,
        max_page_size: typing.Optional[MaxPageSize] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[ExtractorVersionsListResponse]:
        """
        This endpoint allows you to fetch all versions of a given extractor, including the current `draft` version.

        Versions are returned in descending order of creation (newest first) with the `draft` version first. The `draft` version is the latest unpublished version of the extractor, which can be published to create a new version. It might not have any changes from the last published version.

        Parameters
        ----------
        extractor_id : str
            The ID of the extractor.

            Example: `"ex_Xj8mK2pL9nR4vT7qY5wZ"`

        sort_dir : typing.Optional[SortDir]

        next_page_token : typing.Optional[NextPageToken]

        max_page_size : typing.Optional[MaxPageSize]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ExtractorVersionsListResponse]
            Successfully retrieved extractor versions
        """
        _response = self._client_wrapper.httpx_client.request(
            f"extractors/{jsonable_encoder(extractor_id)}/versions",
            method="GET",
            params={
                "sortDir": sort_dir,
                "nextPageToken": next_page_token,
                "maxPageSize": max_page_size,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ExtractorVersionsListResponse,
                    construct_type(
                        type_=ExtractorVersionsListResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 402:
                raise PaymentRequiredError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise core_api_error_ApiError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.text
            )
        raise core_api_error_ApiError(
            status_code=_response.status_code, headers=dict(_response.headers), body=_response_json
        )

    def create(
        self,
        extractor_id: str,
        *,
        release_type: ReleaseType,
        description: typing.Optional[VersionDescription] = OMIT,
        config: typing.Optional[ExtractConfigJsonParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[ExtractorVersion]:
        """
        This endpoint allows you to publish a new version of an existing extractor. Publishing a new version creates a snapshot of the extractor's current configuration and makes it available for use in workflows.

        Publishing a new version does not automatically update existing workflows using this extractor. You may need to manually update workflows to use the new version if desired.

        Parameters
        ----------
        extractor_id : str
            The ID of the extractor.

            Example: `"ex_Xj8mK2pL9nR4vT7qY5wZ"`

        release_type : ReleaseType

        description : typing.Optional[VersionDescription]

        config : typing.Optional[ExtractConfigJsonParams]
            The configuration for this version of the extractor.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ExtractorVersion]
            Successfully published extractor version
        """
        _response = self._client_wrapper.httpx_client.request(
            f"extractors/{jsonable_encoder(extractor_id)}/versions",
            method="POST",
            json={
                "releaseType": release_type,
                "description": description,
                "config": convert_and_respect_annotation_metadata(
                    object_=config, annotation=ExtractConfigJsonParams, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ExtractorVersion,
                    construct_type(
                        type_=ExtractorVersion,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 402:
                raise PaymentRequiredError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise core_api_error_ApiError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.text
            )
        raise core_api_error_ApiError(
            status_code=_response.status_code, headers=dict(_response.headers), body=_response_json
        )

    def retrieve(
        self, extractor_id: str, version_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[ExtractorVersion]:
        """
        Retrieve a specific version of an extractor in Extend

        Parameters
        ----------
        extractor_id : str
            The ID of the extractor.

            Example: `"ex_Xj8mK2pL9nR4vT7qY5wZ"`

        version_id : str
            The ID of the specific extractor version.

            Example: `"extv_QYk6jgHA_8CsO8rVWhyNC"`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ExtractorVersion]
            Successfully retrieved extractor version
        """
        _response = self._client_wrapper.httpx_client.request(
            f"extractors/{jsonable_encoder(extractor_id)}/versions/{jsonable_encoder(version_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ExtractorVersion,
                    construct_type(
                        type_=ExtractorVersion,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 402:
                raise PaymentRequiredError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise core_api_error_ApiError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.text
            )
        raise core_api_error_ApiError(
            status_code=_response.status_code, headers=dict(_response.headers), body=_response_json
        )


class AsyncRawExtractorVersionsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self,
        extractor_id: str,
        *,
        sort_dir: typing.Optional[SortDir] = None,
        next_page_token: typing.Optional[NextPageToken] = None,
        max_page_size: typing.Optional[MaxPageSize] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[ExtractorVersionsListResponse]:
        """
        This endpoint allows you to fetch all versions of a given extractor, including the current `draft` version.

        Versions are returned in descending order of creation (newest first) with the `draft` version first. The `draft` version is the latest unpublished version of the extractor, which can be published to create a new version. It might not have any changes from the last published version.

        Parameters
        ----------
        extractor_id : str
            The ID of the extractor.

            Example: `"ex_Xj8mK2pL9nR4vT7qY5wZ"`

        sort_dir : typing.Optional[SortDir]

        next_page_token : typing.Optional[NextPageToken]

        max_page_size : typing.Optional[MaxPageSize]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ExtractorVersionsListResponse]
            Successfully retrieved extractor versions
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"extractors/{jsonable_encoder(extractor_id)}/versions",
            method="GET",
            params={
                "sortDir": sort_dir,
                "nextPageToken": next_page_token,
                "maxPageSize": max_page_size,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ExtractorVersionsListResponse,
                    construct_type(
                        type_=ExtractorVersionsListResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 402:
                raise PaymentRequiredError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise core_api_error_ApiError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.text
            )
        raise core_api_error_ApiError(
            status_code=_response.status_code, headers=dict(_response.headers), body=_response_json
        )

    async def create(
        self,
        extractor_id: str,
        *,
        release_type: ReleaseType,
        description: typing.Optional[VersionDescription] = OMIT,
        config: typing.Optional[ExtractConfigJsonParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[ExtractorVersion]:
        """
        This endpoint allows you to publish a new version of an existing extractor. Publishing a new version creates a snapshot of the extractor's current configuration and makes it available for use in workflows.

        Publishing a new version does not automatically update existing workflows using this extractor. You may need to manually update workflows to use the new version if desired.

        Parameters
        ----------
        extractor_id : str
            The ID of the extractor.

            Example: `"ex_Xj8mK2pL9nR4vT7qY5wZ"`

        release_type : ReleaseType

        description : typing.Optional[VersionDescription]

        config : typing.Optional[ExtractConfigJsonParams]
            The configuration for this version of the extractor.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ExtractorVersion]
            Successfully published extractor version
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"extractors/{jsonable_encoder(extractor_id)}/versions",
            method="POST",
            json={
                "releaseType": release_type,
                "description": description,
                "config": convert_and_respect_annotation_metadata(
                    object_=config, annotation=ExtractConfigJsonParams, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ExtractorVersion,
                    construct_type(
                        type_=ExtractorVersion,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 402:
                raise PaymentRequiredError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise core_api_error_ApiError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.text
            )
        raise core_api_error_ApiError(
            status_code=_response.status_code, headers=dict(_response.headers), body=_response_json
        )

    async def retrieve(
        self, extractor_id: str, version_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[ExtractorVersion]:
        """
        Retrieve a specific version of an extractor in Extend

        Parameters
        ----------
        extractor_id : str
            The ID of the extractor.

            Example: `"ex_Xj8mK2pL9nR4vT7qY5wZ"`

        version_id : str
            The ID of the specific extractor version.

            Example: `"extv_QYk6jgHA_8CsO8rVWhyNC"`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ExtractorVersion]
            Successfully retrieved extractor version
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"extractors/{jsonable_encoder(extractor_id)}/versions/{jsonable_encoder(version_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ExtractorVersion,
                    construct_type(
                        type_=ExtractorVersion,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 402:
                raise PaymentRequiredError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise core_api_error_ApiError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.text
            )
        raise core_api_error_ApiError(
            status_code=_response.status_code, headers=dict(_response.headers), body=_response_json
        )



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..requests.extract_config_json import ExtractConfigJsonParams
from ..types.extractor import Extractor
from ..types.max_page_size import MaxPageSize
from ..types.next_page_token import NextPageToken
from ..types.sort_by import SortBy
from ..types.sort_dir import SortDir
from .raw_client import AsyncRawExtractorsClient, RawExtractorsClient
from .types.extractors_list_response import ExtractorsListResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class ExtractorsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawExtractorsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawExtractorsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawExtractorsClient
        """
        return self._raw_client

    def list(
        self,
        *,
        next_page_token: typing.Optional[NextPageToken] = None,
        max_page_size: typing.Optional[MaxPageSize] = None,
        sort_by: typing.Optional[SortBy] = None,
        sort_dir: typing.Optional[SortDir] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ExtractorsListResponse:
        """
        List all extractors.

        Returns a summary of each extractor. Use `GET /extractors/{id}` to retrieve the full object including `draftVersion`.

        Parameters
        ----------
        next_page_token : typing.Optional[NextPageToken]

        max_page_size : typing.Optional[MaxPageSize]

        sort_by : typing.Optional[SortBy]

        sort_dir : typing.Optional[SortDir]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ExtractorsListResponse
            Successfully retrieved extractors

        Examples
        --------
        from extend_ai import Extend

        client = Extend(
            token="YOUR_TOKEN",
        )
        client.extractors.list(
            next_page_token="xK9mLPqRtN3vS8wF5hB2cQ==:zWvUxYjM4nKpL7aDgE9HbTcR2mAyX3/Q+CNkfBSw1dZ=",
        )
        """
        _response = self._raw_client.list(
            next_page_token=next_page_token,
            max_page_size=max_page_size,
            sort_by=sort_by,
            sort_dir=sort_dir,
            request_options=request_options,
        )
        return _response.data

    def create(
        self,
        *,
        name: str,
        clone_extractor_id: typing.Optional[str] = OMIT,
        config: typing.Optional[ExtractConfigJsonParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Extractor:
        """
        Create a new extractor.

        Parameters
        ----------
        name : str
            The name of the extractor.

        clone_extractor_id : typing.Optional[str]
            The ID of an existing extractor to clone. If provided, the new extractor will be created with the same config as the extractor with this ID. Cannot be provided together with `config`.

            Example: `"ex_BMdfq_yWM3sT-ZzvCnA3f"`

        config : typing.Optional[ExtractConfigJsonParams]
            The configuration for the extractor. Cannot be provided together with `cloneExtractorId`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Extractor
            Extractor created successfully

        Examples
        --------
        from extend_ai import Extend

        client = Extend(
            token="YOUR_TOKEN",
        )
        client.extractors.create(
            name="Invoice Extractor",
            config={
                "schema": {
                    "type": "object",
                    "properties": {
                        "vendor_name": {
                            "type": "string",
                            "description": "The name of the vendor",
                        },
                        "invoice_number": {
                            "type": "string",
                            "description": "The invoice number",
                        },
                        "total_amount": {
                            "type": "number",
                            "description": "The total amount due",
                        },
                    },
                }
            },
        )
        """
        _response = self._raw_client.create(
            name=name, clone_extractor_id=clone_extractor_id, config=config, request_options=request_options
        )
        return _response.data

    def retrieve(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> Extractor:
        """
        Get details of an extractor.

        Parameters
        ----------
        id : str
            The ID of the extractor to get.

            Example: `"ex_Xj8mK2pL9nR4vT7qY5wZ"`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Extractor
            Extractor details

        Examples
        --------
        from extend_ai import Extend

        client = Extend(
            token="YOUR_TOKEN",
        )
        client.extractors.retrieve(
            id="extractor_id_here",
        )
        """
        _response = self._raw_client.retrieve(id, request_options=request_options)
        return _response.data

    def update(
        self,
        id: str,
        *,
        name: typing.Optional[str] = OMIT,
        config: typing.Optional[ExtractConfigJsonParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Extractor:
        """
        Update an existing extractor.

        Parameters
        ----------
        id : str
            The ID of the extractor to update.

            Example: `"ex_Xj8mK2pL9nR4vT7qY5wZ"`

        name : typing.Optional[str]
            The new name of the extractor.

        config : typing.Optional[ExtractConfigJsonParams]
            The new configuration for the extractor. This will update the draft version of the extractor.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Extractor
            Extractor updated successfully

        Examples
        --------
        from extend_ai import Extend

        client = Extend(
            token="YOUR_TOKEN",
        )
        client.extractors.update(
            id="extractor_id_here",
            name="Invoice Extractor v2",
        )
        """
        _response = self._raw_client.update(id, name=name, config=config, request_options=request_options)
        return _response.data


class AsyncExtractorsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawExtractorsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawExtractorsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawExtractorsClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        next_page_token: typing.Optional[NextPageToken] = None,
        max_page_size: typing.Optional[MaxPageSize] = None,
        sort_by: typing.Optional[SortBy] = None,
        sort_dir: typing.Optional[SortDir] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ExtractorsListResponse:
        """
        List all extractors.

        Returns a summary of each extractor. Use `GET /extractors/{id}` to retrieve the full object including `draftVersion`.

        Parameters
        ----------
        next_page_token : typing.Optional[NextPageToken]

        max_page_size : typing.Optional[MaxPageSize]

        sort_by : typing.Optional[SortBy]

        sort_dir : typing.Optional[SortDir]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ExtractorsListResponse
            Successfully retrieved extractors

        Examples
        --------
        import asyncio

        from extend_ai import AsyncExtend

        client = AsyncExtend(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.extractors.list(
                next_page_token="xK9mLPqRtN3vS8wF5hB2cQ==:zWvUxYjM4nKpL7aDgE9HbTcR2mAyX3/Q+CNkfBSw1dZ=",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.list(
            next_page_token=next_page_token,
            max_page_size=max_page_size,
            sort_by=sort_by,
            sort_dir=sort_dir,
            request_options=request_options,
        )
        return _response.data

    async def create(
        self,
        *,
        name: str,
        clone_extractor_id: typing.Optional[str] = OMIT,
        config: typing.Optional[ExtractConfigJsonParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Extractor:
        """
        Create a new extractor.

        Parameters
        ----------
        name : str
            The name of the extractor.

        clone_extractor_id : typing.Optional[str]
            The ID of an existing extractor to clone. If provided, the new extractor will be created with the same config as the extractor with this ID. Cannot be provided together with `config`.

            Example: `"ex_BMdfq_yWM3sT-ZzvCnA3f"`

        config : typing.Optional[ExtractConfigJsonParams]
            The configuration for the extractor. Cannot be provided together with `cloneExtractorId`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Extractor
            Extractor created successfully

        Examples
        --------
        import asyncio

        from extend_ai import AsyncExtend

        client = AsyncExtend(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.extractors.create(
                name="Invoice Extractor",
                config={
                    "schema": {
                        "type": "object",
                        "properties": {
                            "vendor_name": {
                                "type": "string",
                                "description": "The name of the vendor",
                            },
                            "invoice_number": {
                                "type": "string",
                                "description": "The invoice number",
                            },
                            "total_amount": {
                                "type": "number",
                                "description": "The total amount due",
                            },
                        },
                    }
                },
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(
            name=name, clone_extractor_id=clone_extractor_id, config=config, request_options=request_options
        )
        return _response.data

    async def retrieve(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> Extractor:
        """
        Get details of an extractor.

        Parameters
        ----------
        id : str
            The ID of the extractor to get.

            Example: `"ex_Xj8mK2pL9nR4vT7qY5wZ"`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Extractor
            Extractor details

        Examples
        --------
        import asyncio

        from extend_ai import AsyncExtend

        client = AsyncExtend(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.extractors.retrieve(
                id="extractor_id_here",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.retrieve(id, request_options=request_options)
        return _response.data

    async def update(
        self,
        id: str,
        *,
        name: typing.Optional[str] = OMIT,
        config: typing.Optional[ExtractConfigJsonParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Extractor:
        """
        Update an existing extractor.

        Parameters
        ----------
        id : str
            The ID of the extractor to update.

            Example: `"ex_Xj8mK2pL9nR4vT7qY5wZ"`

        name : typing.Optional[str]
            The new name of the extractor.

        config : typing.Optional[ExtractConfigJsonParams]
            The new configuration for the extractor. This will update the draft version of the extractor.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Extractor
            Extractor updated successfully

        Examples
        --------
        import asyncio

        from extend_ai import AsyncExtend

        client = AsyncExtend(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.extractors.update(
                id="extractor_id_here",
                name="Invoice Extractor v2",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update(id, name=name, config=config, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import typing_extensions
from .webhook_payload_options import WebhookPayloadOptionsParams


class WebhookAdvancedOptionsParams(typing_extensions.TypedDict):
    """
    Advanced configuration options for the webhook endpoint, including custom headers and payload delivery settings.
    """

    headers: typing_extensions.NotRequired[typing.Dict[str, str]]
    """
    Custom HTTP headers to include in webhook requests.
    """

    payload: typing_extensions.NotRequired[WebhookPayloadOptionsParams]



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, update_forward_refs
from ..core.serialization import FieldMetadata
from ..core.unchecked_base_model import UncheckedBaseModel
from .extract_run import ExtractRun


class ConditionalExtractResult(UncheckedBaseModel):
    """
    The result of a conditional extract step, which routes to different extractors based on rules.
    """

    extract_run: typing_extensions.Annotated[ExtractRun, FieldMetadata(alias="extractRun")] = pydantic.Field(
        alias="extractRun"
    )
    """
    The extract run that was executed based on the matched rule.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


update_forward_refs(ConditionalExtractResult)



# This file was auto-generated by Fern from our API Definition.

import typing_extensions


class PageRangesItemParams(typing_extensions.TypedDict):
    start: int
    """
    The start page of the range.
    """

    end: int
    """
    The end page of the range.
    """



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core import enum

T_Result = typing.TypeVar("T_Result")


class ExtractAdvancedOptionsArrayCitationStrategy(enum.StrEnum):
    """
    Granularity for array citations. This requires citationsEnabled=true and a base processor version that supports property-level array citations (extraction_performance ≥ 4.4.0).
    """

    ITEM = "item"
    PROPERTY = "property"
    _UNKNOWN = "__EXTRACTADVANCEDOPTIONSARRAYCITATIONSTRATEGY_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "ExtractAdvancedOptionsArrayCitationStrategy":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        item: typing.Callable[[], T_Result],
        property: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is ExtractAdvancedOptionsArrayCitationStrategy.ITEM:
            return item()
        if self is ExtractAdvancedOptionsArrayCitationStrategy.PROPERTY:
            return property()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import typing_extensions
from ..core.serialization import FieldMetadata
from ..types.block_type import BlockType
from .block_details import BlockDetailsParams
from .block_metadata import BlockMetadataParams
from .block_polygon_item import BlockPolygonItemParams
from .bounding_box import BoundingBoxParams


class BlockParams(typing_extensions.TypedDict):
    object: typing.Literal["block"]
    """
    The type of object. In this case, it will always be `"block"`.
    """

    id: str
    """
    A unique identifier for the block, deterministically generated as a hash of the block content.
    """

    parent_block_id: typing_extensions.NotRequired[
        typing_extensions.Annotated[str, FieldMetadata(alias="parentBlockId")]
    ]
    """
    The ID of the parent block. For example, for a table cell block, this would be the ID of the parent table block. Only set if this is a child block.
    """

    type: BlockType
    """
    The type of block:
    * `"text"` - Regular text content
    * `"heading"` - Section or document headings 
    * `"section_heading"` - Subsection headings
    * `"table"` - Tabular data with rows and columns
    * `"table_head"` - Table header cells
    * `"table_cell"` - Table body cells
    * `"figure"` - Images, charts, diagrams, or logos
    """

    content: str
    """
    The textual content of the block formatted based on the target format.
    """

    details: BlockDetailsParams
    """
    Additional details specific to the block type. The schema depends on the block type.
    """

    metadata: BlockMetadataParams
    """
    Metadata about the block.
    """

    polygon: typing.Sequence[BlockPolygonItemParams]
    """
    An array of points defining the polygon that bounds the block.
    """

    bounding_box: typing_extensions.Annotated[BoundingBoxParams, FieldMetadata(alias="boundingBox")]
    """
    A simplified bounding box for the block.
    """

    children: typing_extensions.NotRequired[typing.Sequence["BlockParams"]]
    """
    An array of child blocks. For example, a table block may contain table cell blocks as children when `cellBlocksEnabled` is set to true.
    """



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core import enum

T_Result = typing.TypeVar("T_Result")


class ParseConfigTarget(enum.StrEnum):
    """
    The target format for the parsed content.

    Supported values:

    * `markdown`: True markdown with logical reading order (headings, lists, tables, checkboxes). Best default for LLMs/RAG and enables section-based chunking.
    * `spatial`: Layout/position-preserving text that uses markdown elements for block types but is not strictly markdown due to whitespace/tabs used to maintain placement. Only page-based chunking is supported.

    Guidance:

    * Prefer `markdown` for most documents, multi-column reading order, and retrieval use cases
    * Prefer `spatial` for messy/scanned/handwritten or skewed documents, when you need near 1:1 layout fidelity, or for BOL-like logistics docs

    See “Markdown vs Spatial” in the [Parse guide](https://docs.extend.ai/2026-02-09/product/parsing/configuration-options#target-format) for details.
    """

    MARKDOWN = "markdown"
    SPATIAL = "spatial"
    _UNKNOWN = "__PARSECONFIGTARGET_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "ParseConfigTarget":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        markdown: typing.Callable[[], T_Result],
        spatial: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is ParseConfigTarget.MARKDOWN:
            return markdown()
        if self is ParseConfigTarget.SPATIAL:
            return spatial()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..requests.webhook_advanced_options import WebhookAdvancedOptionsParams
from ..types.api_version_enum import ApiVersionEnum
from ..types.max_page_size import MaxPageSize
from ..types.next_page_token import NextPageToken
from ..types.sort_dir import SortDir
from ..types.webhook_endpoint import WebhookEndpoint
from ..types.webhook_endpoint_create import WebhookEndpointCreate
from ..types.webhook_endpoint_event_type import WebhookEndpointEventType
from ..types.webhook_endpoint_status import WebhookEndpointStatus
from .raw_client import AsyncRawWebhookEndpointsClient, RawWebhookEndpointsClient
from .types.webhook_endpoints_delete_response import WebhookEndpointsDeleteResponse
from .types.webhook_endpoints_list_response import WebhookEndpointsListResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class WebhookEndpointsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawWebhookEndpointsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawWebhookEndpointsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawWebhookEndpointsClient
        """
        return self._raw_client

    def list(
        self,
        *,
        status: typing.Optional[WebhookEndpointStatus] = None,
        sort_dir: typing.Optional[SortDir] = None,
        next_page_token: typing.Optional[NextPageToken] = None,
        max_page_size: typing.Optional[MaxPageSize] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> WebhookEndpointsListResponse:
        """
        List all webhook endpoints.

        Parameters
        ----------
        status : typing.Optional[WebhookEndpointStatus]
            Filter by endpoint status.

        sort_dir : typing.Optional[SortDir]

        next_page_token : typing.Optional[NextPageToken]

        max_page_size : typing.Optional[MaxPageSize]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        WebhookEndpointsListResponse
            List of webhook endpoints

        Examples
        --------
        from extend_ai import Extend

        client = Extend(
            token="YOUR_TOKEN",
        )
        client.webhook_endpoints.list(
            next_page_token="xK9mLPqRtN3vS8wF5hB2cQ==:zWvUxYjM4nKpL7aDgE9HbTcR2mAyX3/Q+CNkfBSw1dZ=",
        )
        """
        _response = self._raw_client.list(
            status=status,
            sort_dir=sort_dir,
            next_page_token=next_page_token,
            max_page_size=max_page_size,
            request_options=request_options,
        )
        return _response.data

    def create(
        self,
        *,
        url: str,
        name: str,
        enabled_events: typing.Sequence[WebhookEndpointEventType],
        api_version: ApiVersionEnum,
        status: typing.Optional[WebhookEndpointStatus] = OMIT,
        advanced_options: typing.Optional[WebhookAdvancedOptionsParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> WebhookEndpointCreate:
        """
        Create a new webhook endpoint. The response includes a `signingSecret` that is only returned once — store it securely for verifying webhook signatures.

        The `enabledEvents` array specifies which global event types this endpoint should receive. Use the [Webhook Events](https://docs.extend.ai/2026-02-09/developers/api-reference/webhook-events) reference to see available event types.

        To subscribe to events scoped to a specific resource (e.g., a single extractor or workflow), use [Create Webhook Subscription](https://docs.extend.ai/2026-02-09/developers/api-reference/endpoints/webhook/create-webhook-subscription) after creating the endpoint.

        Parameters
        ----------
        url : str
            The URL that webhook events will be sent to.

        name : str
            A human-readable name for the webhook endpoint.

        enabled_events : typing.Sequence[WebhookEndpointEventType]
            The list of global event types to subscribe to. Pass an empty array to create an endpoint with no global events (useful if you only plan to use resource-scoped subscriptions).

        api_version : ApiVersionEnum

        status : typing.Optional[WebhookEndpointStatus]

        advanced_options : typing.Optional[WebhookAdvancedOptionsParams]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        WebhookEndpointCreate
            Successfully created webhook endpoint

        Examples
        --------
        from extend_ai import ApiVersionEnum, Extend, WebhookEndpointEventType

        client = Extend(
            token="YOUR_TOKEN",
        )
        client.webhook_endpoints.create(
            url="https://example.com/webhooks",
            name="Production webhook",
            enabled_events=[
                WebhookEndpointEventType.EXTRACT_RUN_PROCESSED,
                WebhookEndpointEventType.WORKFLOW_CREATED,
            ],
            api_version=ApiVersionEnum.TWO_THOUSAND_TWENTY_SIX0209,
        )
        """
        _response = self._raw_client.create(
            url=url,
            name=name,
            enabled_events=enabled_events,
            api_version=api_version,
            status=status,
            advanced_options=advanced_options,
            request_options=request_options,
        )
        return _response.data

    def retrieve(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> WebhookEndpoint:
        """
        Retrieve a webhook endpoint by ID.

        Parameters
        ----------
        id : str
            The ID of the webhook endpoint.

            Example: `"wh_Xj8mK2pL9nR4vT7qY5wZ"`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        WebhookEndpoint
            Retrieved webhook endpoint

        Examples
        --------
        from extend_ai import Extend

        client = Extend(
            token="YOUR_TOKEN",
        )
        client.webhook_endpoints.retrieve(
            id="webhook_endpoint_id_here",
        )
        """
        _response = self._raw_client.retrieve(id, request_options=request_options)
        return _response.data

    def update(
        self,
        id: str,
        *,
        url: typing.Optional[str] = OMIT,
        name: typing.Optional[str] = OMIT,
        status: typing.Optional[WebhookEndpointStatus] = OMIT,
        enabled_events: typing.Optional[typing.Sequence[WebhookEndpointEventType]] = OMIT,
        advanced_options: typing.Optional[WebhookAdvancedOptionsParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> WebhookEndpoint:
        """
        Update a webhook endpoint. Only the fields you include in the request body will be updated; omitted fields remain unchanged.

        The `apiVersion` of a webhook endpoint cannot be changed after creation.

        Parameters
        ----------
        id : str
            The ID of the webhook endpoint to update.

            Example: `"wh_Xj8mK2pL9nR4vT7qY5wZ"`

        url : typing.Optional[str]
            The URL that webhook events will be sent to.

        name : typing.Optional[str]
            A human-readable name for the webhook endpoint.

        status : typing.Optional[WebhookEndpointStatus]

        enabled_events : typing.Optional[typing.Sequence[WebhookEndpointEventType]]
            The list of global event types to subscribe to.

        advanced_options : typing.Optional[WebhookAdvancedOptionsParams]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        WebhookEndpoint
            Successfully updated webhook endpoint

        Examples
        --------
        from extend_ai import Extend

        client = Extend(
            token="YOUR_TOKEN",
        )
        client.webhook_endpoints.update(
            id="webhook_endpoint_id_here",
        )
        """
        _response = self._raw_client.update(
            id,
            url=url,
            name=name,
            status=status,
            enabled_events=enabled_events,
            advanced_options=advanced_options,
            request_options=request_options,
        )
        return _response.data

    def delete(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> WebhookEndpointsDeleteResponse:
        """
        Delete a webhook endpoint and all of its subscriptions. This operation is permanent and cannot be undone.

        Parameters
        ----------
        id : str
            The ID of the webhook endpoint to delete.

            Example: `"wh_Xj8mK2pL9nR4vT7qY5wZ"`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        WebhookEndpointsDeleteResponse
            Successfully deleted webhook endpoint

        Examples
        --------
        from extend_ai import Extend

        client = Extend(
            token="YOUR_TOKEN",
        )
        client.webhook_endpoints.delete(
            id="webhook_endpoint_id_here",
        )
        """
        _response = self._raw_client.delete(id, request_options=request_options)
        return _response.data


class AsyncWebhookEndpointsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawWebhookEndpointsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawWebhookEndpointsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawWebhookEndpointsClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        status: typing.Optional[WebhookEndpointStatus] = None,
        sort_dir: typing.Optional[SortDir] = None,
        next_page_token: typing.Optional[NextPageToken] = None,
        max_page_size: typing.Optional[MaxPageSize] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> WebhookEndpointsListResponse:
        """
        List all webhook endpoints.

        Parameters
        ----------
        status : typing.Optional[WebhookEndpointStatus]
            Filter by endpoint status.

        sort_dir : typing.Optional[SortDir]

        next_page_token : typing.Optional[NextPageToken]

        max_page_size : typing.Optional[MaxPageSize]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        WebhookEndpointsListResponse
            List of webhook endpoints

        Examples
        --------
        import asyncio

        from extend_ai import AsyncExtend

        client = AsyncExtend(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.webhook_endpoints.list(
                next_page_token="xK9mLPqRtN3vS8wF5hB2cQ==:zWvUxYjM4nKpL7aDgE9HbTcR2mAyX3/Q+CNkfBSw1dZ=",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.list(
            status=status,
            sort_dir=sort_dir,
            next_page_token=next_page_token,
            max_page_size=max_page_size,
            request_options=request_options,
        )
        return _response.data

    async def create(
        self,
        *,
        url: str,
        name: str,
        enabled_events: typing.Sequence[WebhookEndpointEventType],
        api_version: ApiVersionEnum,
        status: typing.Optional[WebhookEndpointStatus] = OMIT,
        advanced_options: typing.Optional[WebhookAdvancedOptionsParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> WebhookEndpointCreate:
        """
        Create a new webhook endpoint. The response includes a `signingSecret` that is only returned once — store it securely for verifying webhook signatures.

        The `enabledEvents` array specifies which global event types this endpoint should receive. Use the [Webhook Events](https://docs.extend.ai/2026-02-09/developers/api-reference/webhook-events) reference to see available event types.

        To subscribe to events scoped to a specific resource (e.g., a single extractor or workflow), use [Create Webhook Subscription](https://docs.extend.ai/2026-02-09/developers/api-reference/endpoints/webhook/create-webhook-subscription) after creating the endpoint.

        Parameters
        ----------
        url : str
            The URL that webhook events will be sent to.

        name : str
            A human-readable name for the webhook endpoint.

        enabled_events : typing.Sequence[WebhookEndpointEventType]
            The list of global event types to subscribe to. Pass an empty array to create an endpoint with no global events (useful if you only plan to use resource-scoped subscriptions).

        api_version : ApiVersionEnum

        status : typing.Optional[WebhookEndpointStatus]

        advanced_options : typing.Optional[WebhookAdvancedOptionsParams]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        WebhookEndpointCreate
            Successfully created webhook endpoint

        Examples
        --------
        import asyncio

        from extend_ai import ApiVersionEnum, AsyncExtend, WebhookEndpointEventType

        client = AsyncExtend(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.webhook_endpoints.create(
                url="https://example.com/webhooks",
                name="Production webhook",
                enabled_events=[
                    WebhookEndpointEventType.EXTRACT_RUN_PROCESSED,
                    WebhookEndpointEventType.WORKFLOW_CREATED,
                ],
                api_version=ApiVersionEnum.TWO_THOUSAND_TWENTY_SIX0209,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(
            url=url,
            name=name,
            enabled_events=enabled_events,
            api_version=api_version,
            status=status,
            advanced_options=advanced_options,
            request_options=request_options,
        )
        return _response.data

    async def retrieve(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> WebhookEndpoint:
        """
        Retrieve a webhook endpoint by ID.

        Parameters
        ----------
        id : str
            The ID of the webhook endpoint.

            Example: `"wh_Xj8mK2pL9nR4vT7qY5wZ"`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        WebhookEndpoint
            Retrieved webhook endpoint

        Examples
        --------
        import asyncio

        from extend_ai import AsyncExtend

        client = AsyncExtend(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.webhook_endpoints.retrieve(
                id="webhook_endpoint_id_here",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.retrieve(id, request_options=request_options)
        return _response.data

    async def update(
        self,
        id: str,
        *,
        url: typing.Optional[str] = OMIT,
        name: typing.Optional[str] = OMIT,
        status: typing.Optional[WebhookEndpointStatus] = OMIT,
        enabled_events: typing.Optional[typing.Sequence[WebhookEndpointEventType]] = OMIT,
        advanced_options: typing.Optional[WebhookAdvancedOptionsParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> WebhookEndpoint:
        """
        Update a webhook endpoint. Only the fields you include in the request body will be updated; omitted fields remain unchanged.

        The `apiVersion` of a webhook endpoint cannot be changed after creation.

        Parameters
        ----------
        id : str
            The ID of the webhook endpoint to update.

            Example: `"wh_Xj8mK2pL9nR4vT7qY5wZ"`

        url : typing.Optional[str]
            The URL that webhook events will be sent to.

        name : typing.Optional[str]
            A human-readable name for the webhook endpoint.

        status : typing.Optional[WebhookEndpointStatus]

        enabled_events : typing.Optional[typing.Sequence[WebhookEndpointEventType]]
            The list of global event types to subscribe to.

        advanced_options : typing.Optional[WebhookAdvancedOptionsParams]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        WebhookEndpoint
            Successfully updated webhook endpoint

        Examples
        --------
        import asyncio

        from extend_ai import AsyncExtend

        client = AsyncExtend(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.webhook_endpoints.update(
                id="webhook_endpoint_id_here",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update(
            id,
            url=url,
            name=name,
            status=status,
            enabled_events=enabled_events,
            advanced_options=advanced_options,
            request_options=request_options,
        )
        return _response.data

    async def delete(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> WebhookEndpointsDeleteResponse:
        """
        Delete a webhook endpoint and all of its subscriptions. This operation is permanent and cannot be undone.

        Parameters
        ----------
        id : str
            The ID of the webhook endpoint to delete.

            Example: `"wh_Xj8mK2pL9nR4vT7qY5wZ"`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        WebhookEndpointsDeleteResponse
            Successfully deleted webhook endpoint

        Examples
        --------
        import asyncio

        from extend_ai import AsyncExtend

        client = AsyncExtend(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.webhook_endpoints.delete(
                id="webhook_endpoint_id_here",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(id, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core import enum

T_Result = typing.TypeVar("T_Result")


class ReleaseType(enum.StrEnum):
    """
    The type of release for this version. The two options are "major" and "minor", which will increment the version number accordingly.
    """

    MAJOR = "major"
    MINOR = "minor"
    _UNKNOWN = "__RELEASETYPE_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "ReleaseType":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        major: typing.Callable[[], T_Result],
        minor: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is ReleaseType.MAJOR:
            return major()
        if self is ReleaseType.MINOR:
            return minor()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from ..core.serialization import FieldMetadata
from ..types.extract_chunking_options_chunk_selection_strategy import ExtractChunkingOptionsChunkSelectionStrategy
from ..types.extract_chunking_options_chunking_strategy import ExtractChunkingOptionsChunkingStrategy


class ExtractChunkingOptionsParams(typing_extensions.TypedDict):
    chunking_strategy: typing_extensions.NotRequired[
        typing_extensions.Annotated[ExtractChunkingOptionsChunkingStrategy, FieldMetadata(alias="chunkingStrategy")]
    ]
    """
    The strategy to use for chunking the document.
    """

    page_chunk_size: typing_extensions.NotRequired[
        typing_extensions.Annotated[int, FieldMetadata(alias="pageChunkSize")]
    ]
    """
    The size of page chunks.
    """

    chunk_selection_strategy: typing_extensions.NotRequired[
        typing_extensions.Annotated[
            ExtractChunkingOptionsChunkSelectionStrategy, FieldMetadata(alias="chunkSelectionStrategy")
        ]
    ]
    """
    The strategy to use for selecting chunks.
    """

    custom_semantic_chunking_rules: typing_extensions.NotRequired[
        typing_extensions.Annotated[str, FieldMetadata(alias="customSemanticChunkingRules")]
    ]
    """
    Custom rules for semantic chunking.
    """



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.serialization import FieldMetadata
from ..core.unchecked_base_model import UncheckedBaseModel
from .legacy_classification import LegacyClassification
from .legacy_splitter_advanced_options import LegacySplitterAdvancedOptions
from .legacy_splitter_config_base_processor import LegacySplitterConfigBaseProcessor
from .parse_config import ParseConfig


class LegacySplitterConfig(UncheckedBaseModel):
    base_processor: typing_extensions.Annotated[
        typing.Optional[LegacySplitterConfigBaseProcessor], FieldMetadata(alias="baseProcessor")
    ] = pydantic.Field(alias="baseProcessor", default=None)
    """
    The base processor to use. For splitters, this can currently only be `"splitting_performance"` or `"splitting_light"`. See [Splitting Changelog](https://docs.extend.ai/2026-02-09/changelog/splitting/splitting-performance) for more details.
    """

    base_version: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="baseVersion")] = (
        pydantic.Field(alias="baseVersion", default=None)
    )
    """
    The version of the `"splitting_performance"` or `"splitting_light"` processor to use. If this is provided, the `baseProcessor` must also be provided. See [Splitting Changelog](https://docs.extend.ai/2026-02-09/changelog/splitting/splitting-performance) for more details.
    """

    split_classifications: typing_extensions.Annotated[
        typing.List[LegacyClassification], FieldMetadata(alias="splitClassifications")
    ] = pydantic.Field(alias="splitClassifications")
    """
    Array of classifications that define the possible types of document sections.
    """

    split_rules: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="splitRules")] = pydantic.Field(
        alias="splitRules", default=None
    )
    """
    Custom rules to guide the document splitting process in natural language.
    """

    advanced_options: typing_extensions.Annotated[
        typing.Optional[LegacySplitterAdvancedOptions], FieldMetadata(alias="advancedOptions")
    ] = pydantic.Field(alias="advancedOptions", default=None)
    """
    Advanced configuration options.
    """

    parser: typing.Optional[ParseConfig] = pydantic.Field(default=None)
    """
    Configuration options for the parsing process.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import typing_extensions
from ..core.serialization import FieldMetadata
from .legacy_json_output_metadata_value_citations_item_polygon_item import (
    LegacyJsonOutputMetadataValueCitationsItemPolygonItemParams,
)


class LegacyJsonOutputMetadataValueCitationsItemParams(typing_extensions.TypedDict):
    page: typing_extensions.NotRequired[float]
    """
    Page number where the citation was found
    """

    reference_text: typing_extensions.NotRequired[
        typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="referenceText")]
    ]
    """
    The text that was referenced
    """

    polygon: typing_extensions.NotRequired[typing.Sequence[LegacyJsonOutputMetadataValueCitationsItemPolygonItemParams]]
    """
    Array of points defining the polygon around the referenced text
    """

    page_width: typing_extensions.NotRequired[typing_extensions.Annotated[float, FieldMetadata(alias="pageWidth")]]
    """
    Width of the page in points
    """

    page_height: typing_extensions.NotRequired[typing_extensions.Annotated[float, FieldMetadata(alias="pageHeight")]]
    """
    Height of the page in points
    """



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core import enum

T_Result = typing.TypeVar("T_Result")


class ArrayStrategyType(enum.StrEnum):
    """
    The strategy type for handling large array use cases. For most use cases this should be left null, or should be set to `large_array_heuristics`. If you're unsure of what your use case needs, reach out to the Extend team for help.
    - `large_array_heuristics`: Optimized for documents with very large arrays, but where latency matters.

      This strategy uses specialized heuristics around chunking, tables, and merging to handle large arrays efficiently over large documents.

    - `large_array_max_context`: Optimizes for accuracy over latency in documents with very large arrays.

      This strategy will do multiple passes through the entire document to ensure there is no context loss across any chunks/pages, maximizing accuracy for complex array extraction, but adding material latency.

    - `large_array_overlap_context`: Balances accuracy and latency in documents with very large arrays.

      This strategy will always maintain surrounding/overlapping page context for every chunk/page that is extracted, eliminating array failure modes from context loss across page boundaries.
    """

    LARGE_ARRAY_HEURISTICS = "large_array_heuristics"
    LARGE_ARRAY_MAX_CONTEXT = "large_array_max_context"
    LARGE_ARRAY_OVERLAP_CONTEXT = "large_array_overlap_context"
    _UNKNOWN = "__ARRAYSTRATEGYTYPE_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "ArrayStrategyType":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        large_array_heuristics: typing.Callable[[], T_Result],
        large_array_max_context: typing.Callable[[], T_Result],
        large_array_overlap_context: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is ArrayStrategyType.LARGE_ARRAY_HEURISTICS:
            return large_array_heuristics()
        if self is ArrayStrategyType.LARGE_ARRAY_MAX_CONTEXT:
            return large_array_max_context()
        if self is ArrayStrategyType.LARGE_ARRAY_OVERLAP_CONTEXT:
            return large_array_overlap_context()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

import typing

import typing_extensions
from ..core.serialization import FieldMetadata


class EvaluationSetRunOptionsParams(typing_extensions.TypedDict):
    """
    The options for the evaluation set run.
    """

    fuzzy_match_fields: typing_extensions.NotRequired[
        typing_extensions.Annotated[typing.Sequence[str], FieldMetadata(alias="fuzzyMatchFields")]
    ]
    """
    The fields that were fuzzy matched.
    """

    exclude_fields: typing_extensions.NotRequired[
        typing_extensions.Annotated[typing.Sequence[str], FieldMetadata(alias="excludeFields")]
    ]
    """
    The fields that were excluded from the run.
    """



# This file was auto-generated by Fern from our API Definition.

import typing

import typing_extensions
from ...core.serialization import FieldMetadata
from ...requests.evaluation_set_item import EvaluationSetItemParams


class EvaluationSetItemsCreateResponseParams(typing_extensions.TypedDict):
    evaluation_set_items: typing_extensions.Annotated[
        typing.Sequence[EvaluationSetItemParams], FieldMetadata(alias="evaluationSetItems")
    ]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.serialization import FieldMetadata
from ..core.unchecked_base_model import UncheckedBaseModel
from .created_at import CreatedAt
from .file_contents import FileContents
from .file_metadata import FileMetadata
from .file_type import FileType
from .updated_at import UpdatedAt


class File(UncheckedBaseModel):
    object: typing.Literal["file"] = pydantic.Field(default="file")
    """
    The type of object. Will always be `"file"`.
    """

    id: str = pydantic.Field()
    """
    ID for the file.
    
    Example: `"file_xK9mLPqRtN3vS8wF5hB2cQ"`
    """

    name: str = pydantic.Field()
    """
    The name of the file
    
    Example: `"Invoices.pdf"`
    """

    type: typing.Optional[FileType] = pydantic.Field(default=None)
    """
    The type of the file.
    
    **Availability:** Present when the file type could be determined.
    """

    presigned_url: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="presignedUrl")] = (
        pydantic.Field(alias="presignedUrl", default=None)
    )
    """
    A presigned URL to download the file. Expires after 15 minutes.
    
    **Availability:** Present on `GET /files/{id}`. Not present on `POST /files/upload` or when the file is embedded in other resources (e.g., in run responses).
    """

    parent_file_id: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="parentFileId")] = (
        pydantic.Field(alias="parentFileId", default=None)
    )
    """
    ID of the parent file.
    
    **Availability:** Present for files created via a Splitter in a workflow.
    """

    contents: typing.Optional[FileContents] = pydantic.Field(default=None)
    """
    **Deprecated:** Use the `POST /parse_runs` endpoint instead to parse and retrieve file contents. The parse runs endpoint provides more control over parsing configuration and better performance.
    
    The parsed content of the file. This field will only contain data after the file has been parsed via a parse run, extract run, classify run, split run, edit run, or workflow run.
    
    **Availability:** Only present and populated on `GET /files/{id}` when the file has been previously parsed and the corresponding query parameters are set to true. Will be `null` on `POST /files/upload` and for files that haven't been parsed. The structure varies based on file type.
    """

    metadata: FileMetadata
    created_at: typing_extensions.Annotated[CreatedAt, FieldMetadata(alias="createdAt")] = pydantic.Field(
        alias="createdAt"
    )
    updated_at: typing_extensions.Annotated[UpdatedAt, FieldMetadata(alias="updatedAt")] = pydantic.Field(
        alias="updatedAt"
    )

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel


class WorkflowStepBase(UncheckedBaseModel):
    """
    Base properties shared by all workflow step types.
    """

    object: typing.Literal["workflow_step"] = pydantic.Field(default="workflow_step")
    """
    The type of object. In this case, it will always be `"workflow_step"`.
    """

    id: str = pydantic.Field()
    """
    The ID of the workflow step.
    
    Example: `"step_xKm9pNv3qWsY_jL2tR5Dh"`
    """

    name: str = pydantic.Field()
    """
    The name of the workflow step.
    
    Example: `"Validate Invoice Total"`
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from .split_result import SplitResultParams
from .split_step_run_step import SplitStepRunStepParams
from .step_run_base import StepRunBaseParams


class SplitStepRunParams(StepRunBaseParams):
    step: SplitStepRunStepParams
    result: typing.Optional[SplitResultParams]
    """
    The result of this split step. Currently, `result` is always present for returned step runs. It is nullable to allow for future support of returning step runs with non-successful statuses.
    """



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..requests.legacy_processor_run_file_input import LegacyProcessorRunFileInputParams
from ..types.legacy_json_object import LegacyJsonObject
from ..types.legacy_max_page_size import LegacyMaxPageSize
from ..types.legacy_next_page_token import LegacyNextPageToken
from ..types.legacy_processor_id import LegacyProcessorId
from ..types.legacy_processor_status import LegacyProcessorStatus
from ..types.legacy_processor_type import LegacyProcessorType
from ..types.legacy_sort_by_enum import LegacySortByEnum
from ..types.legacy_sort_dir_enum import LegacySortDirEnum
from .raw_client import AsyncRawProcessorRunClient, RawProcessorRunClient
from .requests.processor_run_create_request_config import ProcessorRunCreateRequestConfigParams
from .types.processor_run_cancel_response import ProcessorRunCancelResponse
from .types.processor_run_create_response import ProcessorRunCreateResponse
from .types.processor_run_delete_response import ProcessorRunDeleteResponse
from .types.processor_run_get_response import ProcessorRunGetResponse
from .types.processor_run_list_request_source import ProcessorRunListRequestSource
from .types.processor_run_list_response import ProcessorRunListResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class ProcessorRunClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawProcessorRunClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawProcessorRunClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawProcessorRunClient
        """
        return self._raw_client

    def list(
        self,
        *,
        status: typing.Optional[LegacyProcessorStatus] = None,
        processor_id: typing.Optional[str] = None,
        processor_type: typing.Optional[LegacyProcessorType] = None,
        source_id: typing.Optional[str] = None,
        source: typing.Optional[ProcessorRunListRequestSource] = None,
        file_name_contains: typing.Optional[str] = None,
        sort_by: typing.Optional[LegacySortByEnum] = None,
        sort_dir: typing.Optional[LegacySortDirEnum] = None,
        next_page_token: typing.Optional[LegacyNextPageToken] = None,
        max_page_size: typing.Optional[LegacyMaxPageSize] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ProcessorRunListResponse:
        """
        List runs of a Processor. A ProcessorRun represents a single execution of a processor against a file.

        Parameters
        ----------
        status : typing.Optional[LegacyProcessorStatus]
            Filters processor runs by their status. If not provided, no filter is applied.

             The status of a processor run:
             * `"PENDING"` - The processor run has not started yet
             * `"PROCESSING"` - The processor run is in progress
             * `"PROCESSED"` - The processor run completed successfully
             * `"FAILED"` - The processor run encountered an error
             * `"CANCELLED"` - The processor run was cancelled

        processor_id : typing.Optional[str]
            Filters processor runs by the processor ID. If not provided, runs for all processors are returned.

            Example: `"ex_BMdfq_yWM3sT-ZzvCnA3f"`

        processor_type : typing.Optional[LegacyProcessorType]
            Filters processor runs by the processor type. If not provided, runs for all processor types are returned.

            Example: `"EXTRACT"`

        source_id : typing.Optional[str]
            Filters processor runs by the source ID. The source ID corresponds to the entity that created the processor run.

            Example: `"workflow_run_123"`

        source : typing.Optional[ProcessorRunListRequestSource]
            Filters processor runs by the source that created them. If not provided, runs from all sources are returned.

            The source of the processor run:
            * `"ADMIN"` - Created by admin
            * `"BATCH_PROCESSOR_RUN"` - Created from a batch processor run
            * `"PLAYGROUND"` - Created from playground
            * `"WORKFLOW_CONFIGURATION"` - Created from workflow configuration
            * `"WORKFLOW_RUN"` - Created from a workflow run
            * `"STUDIO"` - Created from Studio
            * `"API"` - Created via API

        file_name_contains : typing.Optional[str]
            Filters processor runs by the name of the file. Only returns processor runs where the file name contains this string.

            Example: `"invoice"`

        sort_by : typing.Optional[LegacySortByEnum]
            Sorts the processor runs by the given field.

        sort_dir : typing.Optional[LegacySortDirEnum]
            Sorts the processor runs in ascending or descending order. Ascending order means the earliest processor run is returned first.

        next_page_token : typing.Optional[LegacyNextPageToken]

        max_page_size : typing.Optional[LegacyMaxPageSize]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ProcessorRunListResponse
            You will get a list of summaries for each processor run. These are shortened versions of the full ProcessorRun object.

            To get the full object, use the [Get ProcessorRun](https://docs.extend.ai/2026-02-09/developers/api-reference/endpoints/legacy/get-processor-run) endpoint.

        Examples
        --------
        from extend_ai import Extend

        client = Extend(
            token="YOUR_TOKEN",
        )
        client.processor_run.list(
            next_page_token="xK9mLPqRtN3vS8wF5hB2cQ==:zWvUxYjM4nKpL7aDgE9HbTcR2mAyX3/Q+CNkfBSw1dZ=",
        )
        """
        _response = self._raw_client.list(
            status=status,
            processor_id=processor_id,
            processor_type=processor_type,
            source_id=source_id,
            source=source,
            file_name_contains=file_name_contains,
            sort_by=sort_by,
            sort_dir=sort_dir,
            next_page_token=next_page_token,
            max_page_size=max_page_size,
            request_options=request_options,
        )
        return _response.data

    def create(
        self,
        *,
        processor_id: LegacyProcessorId,
        version: typing.Optional[str] = OMIT,
        file: typing.Optional[LegacyProcessorRunFileInputParams] = OMIT,
        raw_text: typing.Optional[str] = OMIT,
        sync: typing.Optional[bool] = OMIT,
        priority: typing.Optional[int] = OMIT,
        metadata: typing.Optional[LegacyJsonObject] = OMIT,
        config: typing.Optional[ProcessorRunCreateRequestConfigParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ProcessorRunCreateResponse:
        """
        Run processors (extraction, classification, splitting, etc.) on a given document.

        **Synchronous vs Asynchronous Processing:**
        - **Asynchronous (default)**: Returns immediately with `PROCESSING` status. Use webhooks or polling to get results.
        - **Synchronous**: Set `sync: true` to wait for completion and get final results in the response (5-minute timeout).

        **For asynchronous processing:**
        - You can [configure webhooks](https://docs.extend.ai/2026-02-09/product/webhooks/configuration) to receive notifications when a processor run is complete or failed.
        - Or you can [poll the get endpoint](https://docs.extend.ai/2026-02-09/developers/api-reference/endpoints/legacy/get-processor-run) for updates on the status of the processor run.

        Parameters
        ----------
        processor_id : LegacyProcessorId

        version : typing.Optional[str]
            An optional version of the processor to use. When not supplied, the most recent published version of the processor will be used. Special values include:
            - `"latest"` for the most recent published version. If there are no published versions, the draft version will be used.
            - `"draft"` for the draft version.
            - Specific version numbers corresponding to versions your team has published, e.g. `"1.0"`, `"2.2"`, etc.

        file : typing.Optional[LegacyProcessorRunFileInputParams]
            The file to be processed. One of `file` or `rawText` must be provided. Supported file types can be found [here](https://docs.extend.ai/2026-02-09/product/general/supported-file-types).

        raw_text : typing.Optional[str]
            A raw string to be processed. Can be used in place of file when passing raw text data streams. One of `file` or `rawText` must be provided.

        sync : typing.Optional[bool]
            Whether to run the processor synchronously. When `true`, the request will wait for the processor run to complete and return the final results. When `false` (default), the request returns immediately with a `PROCESSING` status, and you can poll for completion or use webhooks. For production use cases, we recommending leaving sync off and building around an async integration for more resiliency, unless your use case is predictably fast (e.g. sub < 30 seconds) run time or otherwise have integration constraints that require a synchronous API. See [Async Processing](https://docs.extend.ai/2026-02-09/developers/async-processing) for more details.

            **Timeout**: Synchronous requests have a 5-minute timeout. If the processor run takes longer, it will continue processing asynchronously and you can retrieve the results via the GET endpoint.

        priority : typing.Optional[int]
            An optional value used to determine the relative order of ProcessorRuns when rate limiting is in effect. Lower values will be prioritized before higher values.

        metadata : typing.Optional[LegacyJsonObject]
            An optional object that can be passed in to identify the run of the document processor. It will be returned back to you in the response and webhooks.

            To categorize processor runs for billing and usage tracking, include `extend:usage_tags` with an array of string values (e.g., `{"extend:usage_tags": ["production", "team-eng", "customer-123"]}`). Tags must contain only alphanumeric characters, hyphens, and underscores; any special characters will be automatically removed.

        config : typing.Optional[ProcessorRunCreateRequestConfigParams]
            The configuration for the processor run. If this is provided, this config will be used. If not provided, the config for the specific version you provide will be used. The type of configuration must match the processor type.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ProcessorRunCreateResponse
            Successfully created processor run

        Examples
        --------
        from extend_ai import Extend

        client = Extend(
            token="YOUR_TOKEN",
        )
        client.processor_run.create(
            processor_id="processor_id_here",
        )
        """
        _response = self._raw_client.create(
            processor_id=processor_id,
            version=version,
            file=file,
            raw_text=raw_text,
            sync=sync,
            priority=priority,
            metadata=metadata,
            config=config,
            request_options=request_options,
        )
        return _response.data

    def get(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> ProcessorRunGetResponse:
        """
        Retrieve details about a specific processor run, including its status, outputs, and any edits made during review.

        A common use case for this endpoint is to poll for the status and final output of an async processor run when using the [Run Processor](https://docs.extend.ai/2026-02-09/developers/api-reference/endpoints/legacy/create-processor-run) endpoint. For instance, if you do not want to not configure webhooks to receive the output via completion/failure events.

        Parameters
        ----------
        id : str
            The unique identifier for this processor run.

            Example: `"exr_Xj8mK2pL9nR4vT7qY5wZ"`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ProcessorRunGetResponse
            Successfully retrieved processor run

        Examples
        --------
        from extend_ai import Extend

        client = Extend(
            token="YOUR_TOKEN",
        )
        client.processor_run.get(
            id="processor_run_id_here",
        )
        """
        _response = self._raw_client.get(id, request_options=request_options)
        return _response.data

    def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> ProcessorRunDeleteResponse:
        """
        Delete a processor run and all associated data from Extend. This operation is permanent and cannot be undone.

        This endpoint can be used if you'd like to manage data retention on your own rather than automated data retention policies. Or make one-off deletions for your downstream customers.

        Parameters
        ----------
        id : str
            The ID of the processor run to delete.

            Example: `"exr_Xj8mK2pL9nR4vT7qY5wZ"`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ProcessorRunDeleteResponse
            Successfully deleted processor run

        Examples
        --------
        from extend_ai import Extend

        client = Extend(
            token="YOUR_TOKEN",
        )
        client.processor_run.delete(
            id="processor_run_id_here",
        )
        """
        _response = self._raw_client.delete(id, request_options=request_options)
        return _response.data

    def cancel(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> ProcessorRunCancelResponse:
        """
        Cancel a running processor run by its ID. This endpoint allows you to stop a processor run that is currently in progress.

        Note: Only processor runs with a status of `"PROCESSING"` can be cancelled. Processor runs that have already completed, failed, or been cancelled cannot be cancelled again.

        Parameters
        ----------
        id : str
            The unique identifier for the processor run to cancel.

            Example: `"exr_Xj8mK2pL9nR4vT7qY5wZ"`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ProcessorRunCancelResponse
            Successfully cancelled processor run

        Examples
        --------
        from extend_ai import Extend

        client = Extend(
            token="YOUR_TOKEN",
        )
        client.processor_run.cancel(
            id="processor_run_id_here",
        )
        """
        _response = self._raw_client.cancel(id, request_options=request_options)
        return _response.data


class AsyncProcessorRunClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawProcessorRunClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawProcessorRunClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawProcessorRunClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        status: typing.Optional[LegacyProcessorStatus] = None,
        processor_id: typing.Optional[str] = None,
        processor_type: typing.Optional[LegacyProcessorType] = None,
        source_id: typing.Optional[str] = None,
        source: typing.Optional[ProcessorRunListRequestSource] = None,
        file_name_contains: typing.Optional[str] = None,
        sort_by: typing.Optional[LegacySortByEnum] = None,
        sort_dir: typing.Optional[LegacySortDirEnum] = None,
        next_page_token: typing.Optional[LegacyNextPageToken] = None,
        max_page_size: typing.Optional[LegacyMaxPageSize] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ProcessorRunListResponse:
        """
        List runs of a Processor. A ProcessorRun represents a single execution of a processor against a file.

        Parameters
        ----------
        status : typing.Optional[LegacyProcessorStatus]
            Filters processor runs by their status. If not provided, no filter is applied.

             The status of a processor run:
             * `"PENDING"` - The processor run has not started yet
             * `"PROCESSING"` - The processor run is in progress
             * `"PROCESSED"` - The processor run completed successfully
             * `"FAILED"` - The processor run encountered an error
             * `"CANCELLED"` - The processor run was cancelled

        processor_id : typing.Optional[str]
            Filters processor runs by the processor ID. If not provided, runs for all processors are returned.

            Example: `"ex_BMdfq_yWM3sT-ZzvCnA3f"`

        processor_type : typing.Optional[LegacyProcessorType]
            Filters processor runs by the processor type. If not provided, runs for all processor types are returned.

            Example: `"EXTRACT"`

        source_id : typing.Optional[str]
            Filters processor runs by the source ID. The source ID corresponds to the entity that created the processor run.

            Example: `"workflow_run_123"`

        source : typing.Optional[ProcessorRunListRequestSource]
            Filters processor runs by the source that created them. If not provided, runs from all sources are returned.

            The source of the processor run:
            * `"ADMIN"` - Created by admin
            * `"BATCH_PROCESSOR_RUN"` - Created from a batch processor run
            * `"PLAYGROUND"` - Created from playground
            * `"WORKFLOW_CONFIGURATION"` - Created from workflow configuration
            * `"WORKFLOW_RUN"` - Created from a workflow run
            * `"STUDIO"` - Created from Studio
            * `"API"` - Created via API

        file_name_contains : typing.Optional[str]
            Filters processor runs by the name of the file. Only returns processor runs where the file name contains this string.

            Example: `"invoice"`

        sort_by : typing.Optional[LegacySortByEnum]
            Sorts the processor runs by the given field.

        sort_dir : typing.Optional[LegacySortDirEnum]
            Sorts the processor runs in ascending or descending order. Ascending order means the earliest processor run is returned first.

        next_page_token : typing.Optional[LegacyNextPageToken]

        max_page_size : typing.Optional[LegacyMaxPageSize]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ProcessorRunListResponse
            You will get a list of summaries for each processor run. These are shortened versions of the full ProcessorRun object.

            To get the full object, use the [Get ProcessorRun](https://docs.extend.ai/2026-02-09/developers/api-reference/endpoints/legacy/get-processor-run) endpoint.

        Examples
        --------
        import asyncio

        from extend_ai import AsyncExtend

        client = AsyncExtend(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.processor_run.list(
                next_page_token="xK9mLPqRtN3vS8wF5hB2cQ==:zWvUxYjM4nKpL7aDgE9HbTcR2mAyX3/Q+CNkfBSw1dZ=",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.list(
            status=status,
            processor_id=processor_id,
            processor_type=processor_type,
            source_id=source_id,
            source=source,
            file_name_contains=file_name_contains,
            sort_by=sort_by,
            sort_dir=sort_dir,
            next_page_token=next_page_token,
            max_page_size=max_page_size,
            request_options=request_options,
        )
        return _response.data

    async def create(
        self,
        *,
        processor_id: LegacyProcessorId,
        version: typing.Optional[str] = OMIT,
        file: typing.Optional[LegacyProcessorRunFileInputParams] = OMIT,
        raw_text: typing.Optional[str] = OMIT,
        sync: typing.Optional[bool] = OMIT,
        priority: typing.Optional[int] = OMIT,
        metadata: typing.Optional[LegacyJsonObject] = OMIT,
        config: typing.Optional[ProcessorRunCreateRequestConfigParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ProcessorRunCreateResponse:
        """
        Run processors (extraction, classification, splitting, etc.) on a given document.

        **Synchronous vs Asynchronous Processing:**
        - **Asynchronous (default)**: Returns immediately with `PROCESSING` status. Use webhooks or polling to get results.
        - **Synchronous**: Set `sync: true` to wait for completion and get final results in the response (5-minute timeout).

        **For asynchronous processing:**
        - You can [configure webhooks](https://docs.extend.ai/2026-02-09/product/webhooks/configuration) to receive notifications when a processor run is complete or failed.
        - Or you can [poll the get endpoint](https://docs.extend.ai/2026-02-09/developers/api-reference/endpoints/legacy/get-processor-run) for updates on the status of the processor run.

        Parameters
        ----------
        processor_id : LegacyProcessorId

        version : typing.Optional[str]
            An optional version of the processor to use. When not supplied, the most recent published version of the processor will be used. Special values include:
            - `"latest"` for the most recent published version. If there are no published versions, the draft version will be used.
            - `"draft"` for the draft version.
            - Specific version numbers corresponding to versions your team has published, e.g. `"1.0"`, `"2.2"`, etc.

        file : typing.Optional[LegacyProcessorRunFileInputParams]
            The file to be processed. One of `file` or `rawText` must be provided. Supported file types can be found [here](https://docs.extend.ai/2026-02-09/product/general/supported-file-types).

        raw_text : typing.Optional[str]
            A raw string to be processed. Can be used in place of file when passing raw text data streams. One of `file` or `rawText` must be provided.

        sync : typing.Optional[bool]
            Whether to run the processor synchronously. When `true`, the request will wait for the processor run to complete and return the final results. When `false` (default), the request returns immediately with a `PROCESSING` status, and you can poll for completion or use webhooks. For production use cases, we recommending leaving sync off and building around an async integration for more resiliency, unless your use case is predictably fast (e.g. sub < 30 seconds) run time or otherwise have integration constraints that require a synchronous API. See [Async Processing](https://docs.extend.ai/2026-02-09/developers/async-processing) for more details.

            **Timeout**: Synchronous requests have a 5-minute timeout. If the processor run takes longer, it will continue processing asynchronously and you can retrieve the results via the GET endpoint.

        priority : typing.Optional[int]
            An optional value used to determine the relative order of ProcessorRuns when rate limiting is in effect. Lower values will be prioritized before higher values.

        metadata : typing.Optional[LegacyJsonObject]
            An optional object that can be passed in to identify the run of the document processor. It will be returned back to you in the response and webhooks.

            To categorize processor runs for billing and usage tracking, include `extend:usage_tags` with an array of string values (e.g., `{"extend:usage_tags": ["production", "team-eng", "customer-123"]}`). Tags must contain only alphanumeric characters, hyphens, and underscores; any special characters will be automatically removed.

        config : typing.Optional[ProcessorRunCreateRequestConfigParams]
            The configuration for the processor run. If this is provided, this config will be used. If not provided, the config for the specific version you provide will be used. The type of configuration must match the processor type.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ProcessorRunCreateResponse
            Successfully created processor run

        Examples
        --------
        import asyncio

        from extend_ai import AsyncExtend

        client = AsyncExtend(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.processor_run.create(
                processor_id="processor_id_here",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(
            processor_id=processor_id,
            version=version,
            file=file,
            raw_text=raw_text,
            sync=sync,
            priority=priority,
            metadata=metadata,
            config=config,
            request_options=request_options,
        )
        return _response.data

    async def get(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> ProcessorRunGetResponse:
        """
        Retrieve details about a specific processor run, including its status, outputs, and any edits made during review.

        A common use case for this endpoint is to poll for the status and final output of an async processor run when using the [Run Processor](https://docs.extend.ai/2026-02-09/developers/api-reference/endpoints/legacy/create-processor-run) endpoint. For instance, if you do not want to not configure webhooks to receive the output via completion/failure events.

        Parameters
        ----------
        id : str
            The unique identifier for this processor run.

            Example: `"exr_Xj8mK2pL9nR4vT7qY5wZ"`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ProcessorRunGetResponse
            Successfully retrieved processor run

        Examples
        --------
        import asyncio

        from extend_ai import AsyncExtend

        client = AsyncExtend(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.processor_run.get(
                id="processor_run_id_here",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(id, request_options=request_options)
        return _response.data

    async def delete(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ProcessorRunDeleteResponse:
        """
        Delete a processor run and all associated data from Extend. This operation is permanent and cannot be undone.

        This endpoint can be used if you'd like to manage data retention on your own rather than automated data retention policies. Or make one-off deletions for your downstream customers.

        Parameters
        ----------
        id : str
            The ID of the processor run to delete.

            Example: `"exr_Xj8mK2pL9nR4vT7qY5wZ"`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ProcessorRunDeleteResponse
            Successfully deleted processor run

        Examples
        --------
        import asyncio

        from extend_ai import AsyncExtend

        client = AsyncExtend(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.processor_run.delete(
                id="processor_run_id_here",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(id, request_options=request_options)
        return _response.data

    async def cancel(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ProcessorRunCancelResponse:
        """
        Cancel a running processor run by its ID. This endpoint allows you to stop a processor run that is currently in progress.

        Note: Only processor runs with a status of `"PROCESSING"` can be cancelled. Processor runs that have already completed, failed, or been cancelled cannot be cancelled again.

        Parameters
        ----------
        id : str
            The unique identifier for the processor run to cancel.

            Example: `"exr_Xj8mK2pL9nR4vT7qY5wZ"`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ProcessorRunCancelResponse
            Successfully cancelled processor run

        Examples
        --------
        import asyncio

        from extend_ai import AsyncExtend

        client = AsyncExtend(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.processor_run.cancel(
                id="processor_run_id_here",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.cancel(id, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from ...core.serialization import FieldMetadata
from ...requests.provided_processor_output import ProvidedProcessorOutputParams


class WorkflowRunsCreateRequestOutputsItemParams(typing_extensions.TypedDict):
    processor_id: typing_extensions.Annotated[str, FieldMetadata(alias="processorId")]
    """
    The ID of the extractor, classifier, or splitter that the output is associated with.
    Example: `"ex_Xj8mK2pL9nR4vT7qY5wZ"`
    """

    output: ProvidedProcessorOutputParams
    """
    The output to be used for the workflow run. The structure will depend on the processor type. More details can be found on the "output type" page for the corresponding processor, in the guides section.
    """



# This file was auto-generated by Fern from our API Definition.

import typing

import typing_extensions
from .workflow_step_base import WorkflowStepBaseParams


class SplitStepRunStepParams(WorkflowStepBaseParams):
    type: typing_extensions.NotRequired[typing.Literal["SPLIT"]]
    """
    The type of workflow step. Will always be `"SPLIT"` for this step run type.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from ..core.serialization import FieldMetadata
from ..types.legacy_splitter_advanced_options_split_method import LegacySplitterAdvancedOptionsSplitMethod
from .page_ranges import PageRangesParams


class LegacySplitterAdvancedOptionsParams(typing_extensions.TypedDict):
    split_identifier_rules: typing_extensions.NotRequired[
        typing_extensions.Annotated[str, FieldMetadata(alias="splitIdentifierRules")]
    ]
    """
    Custom rules for identifying split points.
    """

    split_method: typing_extensions.NotRequired[
        typing_extensions.Annotated[LegacySplitterAdvancedOptionsSplitMethod, FieldMetadata(alias="splitMethod")]
    ]
    """
    The method to use for splitting documents. `high_precision` is more accurate but slower, while `low_latency` is faster but less precise.
    
    **Deprecated:** For `splitting_light` >= 1.3.0 and `splitting_performance` >= 1.5.0, this field has no impact and is ignored if provided. It is still accepted for compatibility with older integrations.
    """

    split_excel_documents_by_sheet_enabled: typing_extensions.NotRequired[
        typing_extensions.Annotated[bool, FieldMetadata(alias="splitExcelDocumentsBySheetEnabled")]
    ]
    """
    For Excel documents, split by worksheet.
    """

    fixed_page_limit: typing_extensions.NotRequired[
        typing_extensions.Annotated[int, FieldMetadata(alias="fixedPageLimit")]
    ]
    """
    Limit processing to a specific number of pages from the beginning of the document. See [Page Ranges](https://docs.extend.ai/2026-02-09/product/page-ranges).
    """

    page_ranges: typing_extensions.NotRequired[
        typing_extensions.Annotated[PageRangesParams, FieldMetadata(alias="pageRanges")]
    ]



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from .parse_config_block_options_figures import ParseConfigBlockOptionsFiguresParams
from .parse_config_block_options_tables import ParseConfigBlockOptionsTablesParams
from .parse_config_block_options_text import ParseConfigBlockOptionsTextParams


class ParseConfigBlockOptionsParams(typing_extensions.TypedDict):
    """
    Options for controlling how different block types are processed.
    """

    figures: typing_extensions.NotRequired[ParseConfigBlockOptionsFiguresParams]
    """
    Options for figure blocks.
    """

    tables: typing_extensions.NotRequired[ParseConfigBlockOptionsTablesParams]
    """
    Options for table blocks.
    """

    text: typing_extensions.NotRequired[ParseConfigBlockOptionsTextParams]
    """
    Options for text blocks.
    """



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .types import (
        ProcessorVersionCreateRequestConfig,
        ProcessorVersionCreateRequestConfig_Classify,
        ProcessorVersionCreateRequestConfig_Extract,
        ProcessorVersionCreateRequestConfig_Splitter,
        ProcessorVersionCreateRequestReleaseType,
        ProcessorVersionCreateResponse,
        ProcessorVersionGetResponse,
        ProcessorVersionListResponse,
    )
    from .requests import (
        ProcessorVersionCreateRequestConfigParams,
        ProcessorVersionCreateRequestConfig_ClassifyParams,
        ProcessorVersionCreateRequestConfig_ExtractParams,
        ProcessorVersionCreateRequestConfig_SplitterParams,
        ProcessorVersionCreateResponseParams,
        ProcessorVersionGetResponseParams,
        ProcessorVersionListResponseParams,
    )
_dynamic_imports: typing.Dict[str, str] = {
    "ProcessorVersionCreateRequestConfig": ".types",
    "ProcessorVersionCreateRequestConfigParams": ".requests",
    "ProcessorVersionCreateRequestConfig_Classify": ".types",
    "ProcessorVersionCreateRequestConfig_ClassifyParams": ".requests",
    "ProcessorVersionCreateRequestConfig_Extract": ".types",
    "ProcessorVersionCreateRequestConfig_ExtractParams": ".requests",
    "ProcessorVersionCreateRequestConfig_Splitter": ".types",
    "ProcessorVersionCreateRequestConfig_SplitterParams": ".requests",
    "ProcessorVersionCreateRequestReleaseType": ".types",
    "ProcessorVersionCreateResponse": ".types",
    "ProcessorVersionCreateResponseParams": ".requests",
    "ProcessorVersionGetResponse": ".types",
    "ProcessorVersionGetResponseParams": ".requests",
    "ProcessorVersionListResponse": ".types",
    "ProcessorVersionListResponseParams": ".requests",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "ProcessorVersionCreateRequestConfig",
    "ProcessorVersionCreateRequestConfigParams",
    "ProcessorVersionCreateRequestConfig_Classify",
    "ProcessorVersionCreateRequestConfig_ClassifyParams",
    "ProcessorVersionCreateRequestConfig_Extract",
    "ProcessorVersionCreateRequestConfig_ExtractParams",
    "ProcessorVersionCreateRequestConfig_Splitter",
    "ProcessorVersionCreateRequestConfig_SplitterParams",
    "ProcessorVersionCreateRequestReleaseType",
    "ProcessorVersionCreateResponse",
    "ProcessorVersionCreateResponseParams",
    "ProcessorVersionGetResponse",
    "ProcessorVersionGetResponseParams",
    "ProcessorVersionListResponse",
    "ProcessorVersionListResponseParams",
]



# This file was auto-generated by Fern from our API Definition.

import typing

import typing_extensions
from ..core.serialization import FieldMetadata
from .insight import InsightParams
from .legacy_json_output_metadata_value_citations_item import LegacyJsonOutputMetadataValueCitationsItemParams


class LegacyJsonOutputMetadataValueParams(typing_extensions.TypedDict):
    ocr_confidence: typing_extensions.NotRequired[
        typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="ocrConfidence")]
    ]
    """
    Confidence score from OCR processing, if applicable
    """

    logprobs_confidence: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="logprobsConfidence")]
    """
    Confidence score based on model logprobs
    """

    review_agent_score: typing_extensions.NotRequired[
        typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="reviewAgentScore")]
    ]
    """
    A 1-5 score indicating the review agent's confidence in the extracted value.
    - 5: High confidence, no issues detected
    - 4: Good confidence, minor observations
    - 3: Moderate confidence, some uncertainty
    - 2: Low confidence, likely issues
    - 1: Very low confidence, significant problems detected
    
    These scores will be present when the `reviewAgent.enabled` flag is set to `true` in the processor config.
    If the review agent is enabled but a score is not returned for a field, this value will be `null`.
    To learn more, view the [Review Agent Documentation](https://docs.extend.ai/2026-02-09/product/extraction/review-agent)
    """

    citations: typing_extensions.NotRequired[typing.Sequence[LegacyJsonOutputMetadataValueCitationsItemParams]]
    insights: typing_extensions.NotRequired[typing.Sequence[InsightParams]]



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..requests.extract_config_json import ExtractConfigJsonParams
from ..types.extract_run import ExtractRun
from ..types.max_page_size import MaxPageSize
from ..types.next_page_token import NextPageToken
from ..types.processor_run_status import ProcessorRunStatus
from ..types.run_metadata import RunMetadata
from ..types.run_priority import RunPriority
from ..types.run_source import RunSource
from ..types.run_source_id import RunSourceId
from ..types.sort_by import SortBy
from ..types.sort_dir import SortDir
from .raw_client import AsyncRawExtractRunsClient, RawExtractRunsClient
from .requests.extract_runs_create_request_extractor import ExtractRunsCreateRequestExtractorParams
from .requests.extract_runs_create_request_file import ExtractRunsCreateRequestFileParams
from .types.extract_runs_delete_response import ExtractRunsDeleteResponse
from .types.extract_runs_list_response import ExtractRunsListResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class ExtractRunsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawExtractRunsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawExtractRunsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawExtractRunsClient
        """
        return self._raw_client

    def list(
        self,
        *,
        status: typing.Optional[ProcessorRunStatus] = None,
        extractor_id: typing.Optional[str] = None,
        source_id: typing.Optional[RunSourceId] = None,
        source: typing.Optional[RunSource] = None,
        file_name_contains: typing.Optional[str] = None,
        sort_by: typing.Optional[SortBy] = None,
        sort_dir: typing.Optional[SortDir] = None,
        next_page_token: typing.Optional[NextPageToken] = None,
        max_page_size: typing.Optional[MaxPageSize] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ExtractRunsListResponse:
        """
        List all extract runs.

        Returns a summary of each run. Use `GET /extract_runs/{id}` to retrieve the full object including `output` and `config`.

        Parameters
        ----------
        status : typing.Optional[ProcessorRunStatus]

        extractor_id : typing.Optional[str]
            Filters extract runs by the extractor ID. If not provided, all extract runs are returned.

            Example: `"ex_BMdfq_yWM3sT-ZzvCnA3f"`

        source_id : typing.Optional[RunSourceId]
            Filters runs by the source ID.

        source : typing.Optional[RunSource]
            Filters runs by the source that created them. If not provided, runs from all sources are returned.

        file_name_contains : typing.Optional[str]
            Filters runs by the name of the file. Only returns runs where the file name contains this string.

            Example: `"invoice"`

        sort_by : typing.Optional[SortBy]

        sort_dir : typing.Optional[SortDir]

        next_page_token : typing.Optional[NextPageToken]

        max_page_size : typing.Optional[MaxPageSize]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ExtractRunsListResponse
            You will get a list of summaries for each extract run. These are shortened versions of the full extract run object.

            To get the full object, use the [Get Extract Run](https://docs.extend.ai/2026-02-09/developers/api-reference/endpoints/extract/get-extract-run) endpoint.

        Examples
        --------
        from extend_ai import Extend

        client = Extend(
            token="YOUR_TOKEN",
        )
        client.extract_runs.list(
            next_page_token="xK9mLPqRtN3vS8wF5hB2cQ==:zWvUxYjM4nKpL7aDgE9HbTcR2mAyX3/Q+CNkfBSw1dZ=",
        )
        """
        _response = self._raw_client.list(
            status=status,
            extractor_id=extractor_id,
            source_id=source_id,
            source=source,
            file_name_contains=file_name_contains,
            sort_by=sort_by,
            sort_dir=sort_dir,
            next_page_token=next_page_token,
            max_page_size=max_page_size,
            request_options=request_options,
        )
        return _response.data

    def create(
        self,
        *,
        file: ExtractRunsCreateRequestFileParams,
        extractor: typing.Optional[ExtractRunsCreateRequestExtractorParams] = OMIT,
        config: typing.Optional[ExtractConfigJsonParams] = OMIT,
        priority: typing.Optional[RunPriority] = OMIT,
        metadata: typing.Optional[RunMetadata] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ExtractRun:
        """
        Extract structured data from a file using an existing extractor or an inline configuration.

        The request returns immediately with a `PROCESSING` status. Use webhooks or poll the Get Extract Run endpoint for results.

        Parameters
        ----------
        file : ExtractRunsCreateRequestFileParams
            The file to be extracted from. Files can be provided as a URL, Extend file ID, or raw text.

        extractor : typing.Optional[ExtractRunsCreateRequestExtractorParams]
            Reference to an existing extractor. One of `extractor` or `config` must be provided.

        config : typing.Optional[ExtractConfigJsonParams]
            Inline extract configuration. One of `extractor` or `config` must be provided.

        priority : typing.Optional[RunPriority]

        metadata : typing.Optional[RunMetadata]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ExtractRun
            Extract completed successfully

        Examples
        --------
        from extend_ai import Extend

        client = Extend(
            token="YOUR_TOKEN",
        )
        client.extract_runs.create(
            extractor={"id": "ex_1234567890"},
            file={"url": "https://example.com/invoice.pdf"},
        )
        """
        _response = self._raw_client.create(
            file=file,
            extractor=extractor,
            config=config,
            priority=priority,
            metadata=metadata,
            request_options=request_options,
        )
        return _response.data

    def retrieve(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> ExtractRun:
        """
        Retrieve details about a specific extract run, including its status, outputs, and any edits made during review.

        A common use case for this endpoint is to poll for the status and final output of an extract run when using the [Create Extract Run](https://docs.extend.ai/2026-02-09/developers/api-reference/endpoints/extract/create-extract-run) endpoint. For instance, if you do not want to not configure webhooks to receive the output via completion/failure events.

        Parameters
        ----------
        id : str
            The unique identifier for this extract run.

            Example: `"ex_Xj8mK2pL9nR4vT7qY5wZ"`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ExtractRun
            Successfully retrieved extract run

        Examples
        --------
        from extend_ai import Extend

        client = Extend(
            token="YOUR_TOKEN",
        )
        client.extract_runs.retrieve(
            id="extract_run_id_here",
        )
        """
        _response = self._raw_client.retrieve(id, request_options=request_options)
        return _response.data

    def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> ExtractRunsDeleteResponse:
        """
        Delete an extract run and all associated data from Extend. This operation is permanent and cannot be undone.

        This endpoint can be used if you'd like to manage data retention on your own rather than automated data retention policies. Or make one-off deletions for your downstream customers.

        Parameters
        ----------
        id : str
            The ID of the extract run.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ExtractRunsDeleteResponse
            Extractor run deleted successfully

        Examples
        --------
        from extend_ai import Extend

        client = Extend(
            token="YOUR_TOKEN",
        )
        client.extract_runs.delete(
            id="id",
        )
        """
        _response = self._raw_client.delete(id, request_options=request_options)
        return _response.data

    def cancel(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> ExtractRun:
        """
        Cancel an in-progress extract run.

        Note: Only extract runs with a status of `"PROCESSING"` can be cancelled. Extractor runs that have already completed, failed, or been cancelled cannot be cancelled again.

        Parameters
        ----------
        id : str
            The ID of the extract run to cancel.

            Example: `"ex_Xj8mK2pL9nR4vT7qY5wZ"`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ExtractRun
            Extractor run cancelled successfully

        Examples
        --------
        from extend_ai import Extend

        client = Extend(
            token="YOUR_TOKEN",
        )
        client.extract_runs.cancel(
            id="extract_run_id_here",
        )
        """
        _response = self._raw_client.cancel(id, request_options=request_options)
        return _response.data


class AsyncExtractRunsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawExtractRunsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawExtractRunsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawExtractRunsClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        status: typing.Optional[ProcessorRunStatus] = None,
        extractor_id: typing.Optional[str] = None,
        source_id: typing.Optional[RunSourceId] = None,
        source: typing.Optional[RunSource] = None,
        file_name_contains: typing.Optional[str] = None,
        sort_by: typing.Optional[SortBy] = None,
        sort_dir: typing.Optional[SortDir] = None,
        next_page_token: typing.Optional[NextPageToken] = None,
        max_page_size: typing.Optional[MaxPageSize] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ExtractRunsListResponse:
        """
        List all extract runs.

        Returns a summary of each run. Use `GET /extract_runs/{id}` to retrieve the full object including `output` and `config`.

        Parameters
        ----------
        status : typing.Optional[ProcessorRunStatus]

        extractor_id : typing.Optional[str]
            Filters extract runs by the extractor ID. If not provided, all extract runs are returned.

            Example: `"ex_BMdfq_yWM3sT-ZzvCnA3f"`

        source_id : typing.Optional[RunSourceId]
            Filters runs by the source ID.

        source : typing.Optional[RunSource]
            Filters runs by the source that created them. If not provided, runs from all sources are returned.

        file_name_contains : typing.Optional[str]
            Filters runs by the name of the file. Only returns runs where the file name contains this string.

            Example: `"invoice"`

        sort_by : typing.Optional[SortBy]

        sort_dir : typing.Optional[SortDir]

        next_page_token : typing.Optional[NextPageToken]

        max_page_size : typing.Optional[MaxPageSize]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ExtractRunsListResponse
            You will get a list of summaries for each extract run. These are shortened versions of the full extract run object.

            To get the full object, use the [Get Extract Run](https://docs.extend.ai/2026-02-09/developers/api-reference/endpoints/extract/get-extract-run) endpoint.

        Examples
        --------
        import asyncio

        from extend_ai import AsyncExtend

        client = AsyncExtend(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.extract_runs.list(
                next_page_token="xK9mLPqRtN3vS8wF5hB2cQ==:zWvUxYjM4nKpL7aDgE9HbTcR2mAyX3/Q+CNkfBSw1dZ=",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.list(
            status=status,
            extractor_id=extractor_id,
            source_id=source_id,
            source=source,
            file_name_contains=file_name_contains,
            sort_by=sort_by,
            sort_dir=sort_dir,
            next_page_token=next_page_token,
            max_page_size=max_page_size,
            request_options=request_options,
        )
        return _response.data

    async def create(
        self,
        *,
        file: ExtractRunsCreateRequestFileParams,
        extractor: typing.Optional[ExtractRunsCreateRequestExtractorParams] = OMIT,
        config: typing.Optional[ExtractConfigJsonParams] = OMIT,
        priority: typing.Optional[RunPriority] = OMIT,
        metadata: typing.Optional[RunMetadata] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ExtractRun:
        """
        Extract structured data from a file using an existing extractor or an inline configuration.

        The request returns immediately with a `PROCESSING` status. Use webhooks or poll the Get Extract Run endpoint for results.

        Parameters
        ----------
        file : ExtractRunsCreateRequestFileParams
            The file to be extracted from. Files can be provided as a URL, Extend file ID, or raw text.

        extractor : typing.Optional[ExtractRunsCreateRequestExtractorParams]
            Reference to an existing extractor. One of `extractor` or `config` must be provided.

        config : typing.Optional[ExtractConfigJsonParams]
            Inline extract configuration. One of `extractor` or `config` must be provided.

        priority : typing.Optional[RunPriority]

        metadata : typing.Optional[RunMetadata]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ExtractRun
            Extract completed successfully

        Examples
        --------
        import asyncio

        from extend_ai import AsyncExtend

        client = AsyncExtend(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.extract_runs.create(
                extractor={"id": "ex_1234567890"},
                file={"url": "https://example.com/invoice.pdf"},
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(
            file=file,
            extractor=extractor,
            config=config,
            priority=priority,
            metadata=metadata,
            request_options=request_options,
        )
        return _response.data

    async def retrieve(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> ExtractRun:
        """
        Retrieve details about a specific extract run, including its status, outputs, and any edits made during review.

        A common use case for this endpoint is to poll for the status and final output of an extract run when using the [Create Extract Run](https://docs.extend.ai/2026-02-09/developers/api-reference/endpoints/extract/create-extract-run) endpoint. For instance, if you do not want to not configure webhooks to receive the output via completion/failure events.

        Parameters
        ----------
        id : str
            The unique identifier for this extract run.

            Example: `"ex_Xj8mK2pL9nR4vT7qY5wZ"`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ExtractRun
            Successfully retrieved extract run

        Examples
        --------
        import asyncio

        from extend_ai import AsyncExtend

        client = AsyncExtend(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.extract_runs.retrieve(
                id="extract_run_id_here",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.retrieve(id, request_options=request_options)
        return _response.data

    async def delete(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ExtractRunsDeleteResponse:
        """
        Delete an extract run and all associated data from Extend. This operation is permanent and cannot be undone.

        This endpoint can be used if you'd like to manage data retention on your own rather than automated data retention policies. Or make one-off deletions for your downstream customers.

        Parameters
        ----------
        id : str
            The ID of the extract run.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ExtractRunsDeleteResponse
            Extractor run deleted successfully

        Examples
        --------
        import asyncio

        from extend_ai import AsyncExtend

        client = AsyncExtend(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.extract_runs.delete(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(id, request_options=request_options)
        return _response.data

    async def cancel(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> ExtractRun:
        """
        Cancel an in-progress extract run.

        Note: Only extract runs with a status of `"PROCESSING"` can be cancelled. Extractor runs that have already completed, failed, or been cancelled cannot be cancelled again.

        Parameters
        ----------
        id : str
            The ID of the extract run to cancel.

            Example: `"ex_Xj8mK2pL9nR4vT7qY5wZ"`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ExtractRun
            Extractor run cancelled successfully

        Examples
        --------
        import asyncio

        from extend_ai import AsyncExtend

        client = AsyncExtend(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.extract_runs.cancel(
                id="extract_run_id_here",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.cancel(id, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.serialization import FieldMetadata
from ..core.unchecked_base_model import UncheckedBaseModel
from .page_ranges import PageRanges
from .split_advanced_options_split_method import SplitAdvancedOptionsSplitMethod


class SplitAdvancedOptions(UncheckedBaseModel):
    split_identifier_rules: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="splitIdentifierRules")
    ] = pydantic.Field(alias="splitIdentifierRules", default=None)
    """
    Custom rules for identifying split points.
    
    **Deprecated:** For `splitting_light` >= 1.3.0 and `splitting_performance` >= 1.5.0, use the `identifierKey` field on each classification in `splitClassifications` instead. This provides per-type identifier extraction rules rather than a single global rule. On those versions, this field is accepted but ignored if provided.
    """

    split_method: typing_extensions.Annotated[
        typing.Optional[SplitAdvancedOptionsSplitMethod], FieldMetadata(alias="splitMethod")
    ] = pydantic.Field(alias="splitMethod", default=None)
    """
    The method to use for splitting documents. `high_precision` is more accurate but slower, while `basic_precision` is faster but less precise.
    
    **Deprecated:** For `splitting_light` >= 1.3.0 and `splitting_performance` >= 1.5.0, this field has no impact and is ignored if provided. It is still accepted for compatibility with older integrations.
    """

    split_excel_documents_by_sheet_enabled: typing_extensions.Annotated[
        typing.Optional[bool], FieldMetadata(alias="splitExcelDocumentsBySheetEnabled")
    ] = pydantic.Field(alias="splitExcelDocumentsBySheetEnabled", default=None)
    """
    For Excel documents, split by worksheet.
    """

    page_ranges: typing_extensions.Annotated[typing.Optional[PageRanges], FieldMetadata(alias="pageRanges")] = (
        pydantic.Field(alias="pageRanges", default=None)
    )
    page_overlap_enabled: typing_extensions.Annotated[
        typing.Optional[bool], FieldMetadata(alias="pageOverlapEnabled")
    ] = pydantic.Field(alias="pageOverlapEnabled", default=None)
    """
    When enabled, the splitter will allow for page overlap when splitting a document such that a page can occur in multiple adjacent splits when context from the previous split is on the page and context from the next split is on the page.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError as core_api_error_ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.request_options import RequestOptions
from ..core.unchecked_base_model import construct_type
from ..errors.bad_request_error import BadRequestError
from ..errors.forbidden_error import ForbiddenError
from ..errors.internal_server_error import InternalServerError
from ..errors.not_found_error import NotFoundError
from ..errors.payment_required_error import PaymentRequiredError
from ..errors.too_many_requests_error import TooManyRequestsError
from ..errors.unauthorized_error import UnauthorizedError
from ..errors.unprocessable_entity_error import UnprocessableEntityError
from ..types.api_error import ApiError as types_api_error_ApiError
from ..types.workflow import Workflow

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawWorkflowsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def create(self, *, name: str, request_options: typing.Optional[RequestOptions] = None) -> HttpResponse[Workflow]:
        """
        Create a new workflow in Extend. Workflows are sequences of steps that process files and data in a specific order to achieve a desired outcome.

        This endpoint will create a new workflow in Extend, which can then be configured and deployed. Typically, workflows are created from our UI, however this endpoint can be used to create workflows programmatically. Configuration of the flow still needs to be done in the dashboard.

        Parameters
        ----------
        name : str
            The name of the workflow

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[Workflow]
            Successfully created workflow
        """
        _response = self._client_wrapper.httpx_client.request(
            "workflows",
            method="POST",
            json={
                "name": name,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    Workflow,
                    construct_type(
                        type_=Workflow,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 402:
                raise PaymentRequiredError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise core_api_error_ApiError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.text
            )
        raise core_api_error_ApiError(
            status_code=_response.status_code, headers=dict(_response.headers), body=_response_json
        )


class AsyncRawWorkflowsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def create(
        self, *, name: str, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[Workflow]:
        """
        Create a new workflow in Extend. Workflows are sequences of steps that process files and data in a specific order to achieve a desired outcome.

        This endpoint will create a new workflow in Extend, which can then be configured and deployed. Typically, workflows are created from our UI, however this endpoint can be used to create workflows programmatically. Configuration of the flow still needs to be done in the dashboard.

        Parameters
        ----------
        name : str
            The name of the workflow

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[Workflow]
            Successfully created workflow
        """
        _response = await self._client_wrapper.httpx_client.request(
            "workflows",
            method="POST",
            json={
                "name": name,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    Workflow,
                    construct_type(
                        type_=Workflow,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 402:
                raise PaymentRequiredError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        types_api_error_ApiError,
                        construct_type(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise core_api_error_ApiError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.text
            )
        raise core_api_error_ApiError(
            status_code=_response.status_code, headers=dict(_response.headers), body=_response_json
        )
